unit Unit74;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, TeEngine, Series, ExtCtrls, TeeProcs, Chart, StdCtrls, Contnrs;

{x$define CodeSite} 

const
  UM_UPDATE_CHART = WM_USER;


type
  TChartDataItem = class
  private
    FDate: integer;
    FValue: double;
  public
    property Date: integer read FDate;
    property Value: double read FValue;
  end;

  TChartDataItemList = class(TObjectList)
  private
    function GetData: string;
  protected
    function Get(ndx: Integer): TChartDataItem;
    procedure Put(ndx: Integer; newItem: TChartDataItem);
  public
    function IndexOf(AItem: TChartDataItem): Integer;
    function Remove(AItem: TChartDataItem): Integer;
    function Add(AItem: TChartDataItem): Integer; overload;
    function Add(const aDate: integer; const aValue: double): TChartDataItem; overload;
    property Items[ndx: Integer]: TChartDataItem read Get write Put; default;
    property Data: string read GetData;
  end;    { TChartDataItemList }

  TForm74 = class(TForm)
    Chart1: TChart;
    Series1: TAreaSeries;
    Series2: TAreaSeries;
    Panel1: TPanel;
    lbxFiles: TListBox;
    Label1: TLabel;
    procedure lbxFilesDragDrop(Sender, Source: TObject; X, Y: Integer);
    procedure lbxFilesDragOver(Sender, Source: TObject; X, Y: Integer; State:
        TDragState; var Accept: Boolean);
  private
    { Private declarations }
    procedure UpdateFileNames;
    function GetFilePath: string;
    procedure UMUpdateChart(var aMsg: TMessage); message UM_UPDATE_CHART;
    procedure UpdateChart;
    function CreateSeries: TAreaSeries;
    procedure LoadData(const aFileName: String; aData: TChartDataItemList);
  public
    { Public declarations }
    constructor Create(aO: TComponent); override;
  end;

var
  Form74: TForm74;

implementation

{$R *.dfm}

uses Math {$ifdef CodeSite}, CSIntfEx {$endif};


{ TChartDataItemList }

function TChartDataItemList.Add(AItem: TChartDataItem): Integer;
begin
  result := inherited Add(AItem);
end;    { Add }

function TChartDataItemList.Add(const aDate: integer;
  const aValue: double): TChartDataItem;
begin
  result := TChartDataItem.Create;
  result.FDate := aDate;
  result.FValue := aValue;
  Add(result);
end;

function TChartDataItemList.Get(ndx: Integer): TChartDataItem;
begin
  result := TChartDataItem(inherited Get(ndx));
end;

function TChartDataItemList.GetData: string;
var
  I: Integer;
begin
  result := '';
  for I := 0 to Count - 1 do
    result := format('%s'#13#10'%12s,%10.2f', [result, DateToStr(Get(I).Date), Get(I).Value]);
  if length(result) > 0 then
    result := Copy(result, 3, MaxInt);
end;

{ Get }

function TChartDataItemList.IndexOf(AItem: TChartDataItem): Integer;
var
  j: Integer;
begin
  for j := 0 to Count - 1 do
    if Items[j] = AItem then
      begin
        result := j;
        exit;
      end;
  result := - 1;
end;    { IndexOf }

procedure TChartDataItemList.Put(ndx: Integer; newItem: TChartDataItem);
begin
  inherited Put(ndx, newItem);
end;    { Put }

function TChartDataItemList.Remove(AItem: TChartDataItem): Integer;
begin
  result := IndexOf(AItem);
  if result >= 0 then    { Found. }
    Delete(result);  // Item will be freed through Notify method called by inherited Delete.
end;    { Remove }

{ TForm74 }

constructor TForm74.Create(aO: TComponent);
begin
  inherited;

  UpdateFileNames;

end;



function TForm74.GetFilePath: string;
begin
  result := ExtractFilePath(ParamStr(0));
end;

procedure TForm74.lbxFilesDragDrop(Sender, Source: TObject; X, Y: Integer);
var
  lIdx: Integer;
begin
  lIdx := lbxFiles.ItemAtPos(Point(X, Y), True);
  if (lIdx = -1) then
    lIdx := lbxFiles.Items.Count-1;

  if lbxFiles.ItemIndex <> lIdx then
  begin
    lbxFiles.Items.Move(lbxFiles.ItemIndex, lIdx);
    PostMessage(Handle, UM_UPDATE_CHART, 0, 0);
  end;
end;

procedure TForm74.lbxFilesDragOver(Sender, Source: TObject; X, Y: Integer;
    State: TDragState; var Accept: Boolean);
begin
  Accept := (Sender = Source) and (lbxFiles.ItemIndex > -1);
end;


function TForm74.CreateSeries: TAreaSeries;
begin
  result := TAreaSeries.Create(self);
  result.Marks.Arrow.Visible := True;
  result.Marks.Callout.Brush.Color := clBlack;
  result.Marks.Callout.Arrow.Visible := True;
  result.Marks.Visible := False;
  result.DrawArea := True;
  result.MultiArea := maStacked;
  result.Pointer.InflateMargins := True;
  result.Pointer.Style := psRectangle;
  result.Pointer.Visible := False;
  result.XValues.Name := 'X';
  result.XValues.Order := loAscending;
  result.YValues.Name := 'Y';
  result.YValues.Order := loNone;
  result.ParentChart := Chart1;
end;


procedure TForm74.LoadData(const aFileName: String; aData: TChartDataItemList);
var
  lStrs: TStringList;
  I: Integer;
  lDate: Integer;
  lLine: string;
  lPos: Integer;
  lValue: Double;
begin
  aData.Clear;
  
  lStrs := TStringList.Create;
  try
    lStrs.LoadFromFile(GetFilePath + aFileName);
    for I := 0 to lStrs.Count - 1 do
    begin
      lLine := lStrs[I];
      lPos := Pos(#9, lLine);
      lDate := Trunc(StrToDate(Copy(lLine, 1, lPos-1)));
      lValue := StrToFloat(Copy(lLine, lPos + 1, MaxInt));
      aData.Add(lDate, lValue);
    end;

  finally
    lStrs.Free;
  end;

  {$ifdef CodeSite}
  CodeSite.Send('Data for ' + aFileName, aData.Data)
  {$endif}

end;

procedure TForm74.UpdateChart;
var
  I: Integer;
  J: Integer;
  lSrs: TChartSeries;
  Stairs: array of TPoint; //X values will store the series index, and Y values will store the value index of the stairs
  lChartData: TChartDataItemList;
  lItem: TChartDataItem;
  lLastIdx: Integer;
  lSeriesCount: Integer;
begin
  Chart1.FreeAllSeries;

  lChartData := TChartDataItemList.Create;
  try
    SetLength(Stairs,0);

    lSeriesCount := lbxFiles.Items.Count-1;

    for I := 0 to lSeriesCount do
    begin
      LoadData(lbxFiles.Items[I], lChartData);

      lSrs := CreateSeries;
      lSrs.Title := 'Series #' + IntToStr(I+1);
      lLastIdx := lChartData.Count - 1;

      for J := 0 to lLastIdx do
      begin
        lItem := lChartData[J];
        if lItem.Value > 0 then
          lSrs.AddXY(lItem.Date, lItem.Value)
        else
          lSrs.AddNullXY(lItem.Date, 0);

        if (J < lLastIdx) and
          (lItem.Value > 0) and
          (lChartData[J+1].Value = 0) then
        begin
          lSrs.AddNullXY(lItem.Date, 0);
          SetLength(Stairs,Length(Stairs)+1);
          Stairs[Length(Stairs)-1] := Point(I,J);
        end;
      end;
    end;

    for I := Length(Stairs)-1 downto 0 do
      for J := Stairs[I].X+1 to lSeriesCount-1 do
      begin
        lSrs := Chart1[J];
        lSrs.AddXY(lSrs.XValue[Stairs[I].Y], lSrs.YValue[Stairs[I].Y]);
      end;{}
  finally
    lChartData.Free;
  end;
end;


procedure TForm74.UMUpdateChart(var aMsg: TMessage);
begin
  Updatechart;
end;

procedure TForm74.UpdateFileNames;
var
  lPath: string;
  lFileMask: string;
  lSR: TSearchRec;
begin
  lPath := GetFilePath;
  lFileMask := lPath + 'Data??.txt';

  lbxFiles.Items.Clear;
  if FindFirst(lFileMask, faAnyFile, lSR) = 0 then
  begin
    repeat
      if (Copy(lSR.Name, 1, 1) <> '.') then
        lbxFiles.Items.Add(lSR.Name);
    until (FindNext(lSR) <> 0);
    FindClose(lSR);
  end;
  PostMessage(Handle, UM_UPDATE_CHART, 0, 0);
end;

end.
